#ifndef __CINT__

#include "GEM.h"
#include "SBS.h"
#include "SBSsim/SBSsim.h"
#include "SBSsim/SBSsimDecoder.h"
#include "SBSsim/SBSsimRun.h"
#include "TSystem.h"
#include "THaInterface.h"
#include "TList.h"
#include "TString.h"
#include "TFile.h"
#include "THaGlobals.h"
#include "THaApparatus.h"
#include "THaDetector.h"
#include "THaTextvars.h"
#include "THaRun.h"
#include "THaAnalyzer.h"

using namespace TreeSearch;

#endif

void MC_replay() {

  gSystem->Load("libTreeSearch-GEM");
  gSystem->Load("libSBSsim");


  THaInterface::SetDecoder( SBSsimDecoder::Class() );

  THaApparatus* SBS = new TreeSearch::SBS( "SBS", "SBS apparatus" );
  THaDetector*  gem = new TreeSearch::GEM( "gem_xy", "GEM trackers with xy readouts" );
  SBS->AddDetector( gem );
  gHaApps->Add( SBS );

  SBS->SetDebugAll(1);

  TString db_prefix = SBS->GetName();
  db_prefix += ".";
  db_prefix += gem->GetName();

  gHaTextvars->Add( "DET", db_prefix.Data() );
  gHaTextvars->Add( "SBS", SBS->GetName() );

  THaAnalyzer* analyzer = new THaAnalyzer;

  TString rootfile("SBSsim.root"), odeffile("SBSsim.odef");
  analyzer->EnableBenchmarks();
  analyzer->SetOutFile(rootfile);
  analyzer->SetOdefFile(odeffile);
  analyzer->SetCrateMapFileName("SBSsim/db_SBSsim_cratemap.dat");

  //  THaRunBase* run = new SBSsimRun( "MC/g17/0/all.root", "g17 test run" );  

  //  THaRunBase* run = new SBSsimRun( "MC/g17/sa/10/digi_000.root", "g17 test run" );
  
  //  THaRunBase* run = new SBSsimRun( "MC/g17/sa/1/digi_000.root", "g17 test run" );

  THaRunBase* run = new SBSsimRun( "MC/g17/1/digi_000.root", "g17 test run" );

  //  THaRunBase* run = new SBSsimRun( "MC/g18/1/digi_000.root", "g17 test run" );
  
  run->SetLastEvent(1000);
  //      run->SetLastEvent(1);
  //  run->SetLastEvent(1000);
  //  run->SetLastEvent(300);
  //  run->SetEventRange(3,3);
  //  run->SetEventRange(4,4);
  //  run->SetEventRange(252,252);
  //  run->SetEventRange(619,619);
  //    run->SetEventRange(17,17);
  //  run->SetEventRange(9,9);
  //  run->SetEventRange(18,18);
  //  run->SetEventRange(18,20);
  //  run->SetEventRange(24,40);
  //  run->SetEventRange(22,22);
  //  run->SetEventRange(1,10);

  analyzer->Process(run); // analyze this run
  
  analyzer->Close();
  delete run; run = 0;
  delete analyzer; analyzer = 0;
  gHaApps->Delete();

  TFile* f = new TFile(rootfile,"r");

  cout << "##############################" << endl;
  gDirectory->pwd();
  gDirectory->ls();
  cout << "##############################" << endl;
}


