/////////////////////////////////////////////////////////////////////
//
//   SBSsimRun
//
//   Interface to a ROOT file with simulated SBS data
//
//   Jens-Ole Hansen (ole@jlab.org)  July 2010
//
/////////////////////////////////////////////////////////////////////

#ifndef SBSsimRun_
#define SBSsimRun_

#include "THaRunBase.h"
#include "SBSsim.h"
#include "TString.h"

class TFile;
class TTree;
class TBranch;

class SBSsimRun : public THaRunBase {
 public:
  SBSsimRun(const char* filename = "", const char* description = "");
  SBSsimRun(const SBSsimRun &run);
  virtual ~SBSsimRun();
  virtual SBSsimRun &operator=(const THaRunBase &rhs);

  virtual void  Print( Option_t* opt="" ) const;

  Int_t         Close();
  virtual Int_t Compare( const TObject* obj ) const;
  const Int_t*  GetEvBuffer() const;
  Int_t         Init();
  const char*   GetFileName() const { return rootFileName.Data(); }
  Int_t         Open();
  Int_t         ReadEvent();
  void          SetFileName( const char* name ) { rootFileName = name; }

 protected:
  virtual Int_t ReadDatabase() {return 0;}

  enum EFlags { kHasHitpos, kHasTracks };

  TString rootFileName;  //  Name of input file
  TFile *rootFile;       //! Input ROOT file
  TTree *tree;           //! Input Tree with simulation data
  SBSsimEvent *event;    //! Current event

  ULong64_t nentries;    //! Number of entries in tree
  ULong64_t entry;       //! Current entry number

  UInt_t fFlags;         // Status flags (see EFlags)

  ClassDef(SBSsimRun, 1) // Interface to ROOT file with simulated SBS data
};

#endif
