{
  gStyle->SetOptFit(1);

  // Reconstruction uncertainties (from replay)
  const Float_t sigx    = 4.13e-5;
  const Float_t sigy    = 5.79e-5;   // est., not Gaussian
  const Float_t sigth   = 2.05e-4;
  const Float_t sigph   = 2.13e-4;
  
  // Offsets (from replay)
  const Float_t xoff    = 0.0; //-2.01e-4;
  const Float_t yoff    = 0.0; //-3.01e-4;

  // Number of sigmas within which we accept tracks
  const Float_t nsigma  = 3.0;

  // "Kinematics window"
  const Float_t x_kine  = 0.18;
  const Float_t y_kine  = 0.002;
  const Float_t th_kine = 0.03;
  const Float_t ph_kine = 0.0007;

  string inp;
  cout << "Production? " << flush;
  cin >> inp;

  if( inp.empty() || inp[0] != 'g' ) {
    cerr << "Invalid name, must be g#" << endl;
    return;
  }

  //  TString rootfile(Form("SBSsim_%s.root",inp.c_str()));
  TString rootfile(Form("SBSsim.root",inp.c_str()));
  TFile* f = new TFile(rootfile,"r");
  cout << "Open file -> " << rootfile << endl;

  TTree* T = static_cast<TTree*>(f->Get("T"));

  Long64_t nevt = T->GetEntries();
  if( nevt == 0 ) {
    cerr << "No events in tree" << endl;
    return;
  }
  TH1* h_evt = new TH1I("h_evt","evtnum",1000,0.,1001.);
  // ctrk: good track
  TCut ctrk = Form("abs(SBS.tr.x-MC.tr.x-%f)<%f && "
		   "abs(SBS.tr.y-MC.tr.y-%f)<%f && "
		   "abs(SBS.tr.th-MC.tr.th)<%f && "
		   "abs(SBS.tr.ph-MC.tr.ph)<%f",
		   xoff, nsigma*sigx, yoff, nsigma*sigy,
		   //		   sigth, sigph);
		   nsigma*sigth, nsigma*sigph);
  TCut cntrk = "SBS.tr.n>0";
  ctrk.Print();
  T->Draw("Entry$>>h_evt", ctrk && cntrk, "goff" );

  UInt_t c_trk=0, c_multi=0;
  for(Long64_t i=0; i<h_evt->GetNbinsX(); ++i) {
    if( h_evt->GetBinContent(i)>0.1 )
      ++c_trk;
    if( h_evt->GetBinContent(i)>1.1 )
      ++c_multi;
  }
  UInt_t c_single = c_trk - c_multi;
  cout << "Events with >= 1 matching tracks = " << c_trk << endl;
  cout << "Events with == 1 matching track  = " << c_single << endl;
  cout << ">=1 track efficiency = " << double(c_trk)/nevt*100.
       << "%" << endl;
  cout << "==1 track efficiency = " << double(c_single)/nevt*100.
       << "%" << endl;
  cout << "c_multi = " << c_multi << endl;

  // ckine: track within kinematics window
//   TCut ckine = Form("abs(SBS.tr.x-%f)<%f"
// 		    "&&abs(SBS.tr.y-%f)<%f"
// 		    "&&abs(SBS.tr.th)<%f"
// 		    "&&abs(SBS.tr.ph)<%f",
// 		    xoff,x_kine,yoff,y_kine,th_kine,ph_kine);
  TCut ckine = Form("abs(SBS.tr.x-MC.tr.x-%f)<%f"
		    "&&abs(SBS.tr.y-MC.tr.y-%f)<%f"
		    "&&abs(SBS.tr.th-MC.tr.th)<%f"
		    "&&abs(SBS.tr.ph-MC.tr.ph)<%f",
		    xoff,x_kine,yoff,y_kine,th_kine,ph_kine);
  ckine.Print();
  h_evt->Reset();
  UInt_t c_kine = 0;
  T->Draw("Entry$>>h_evt", ckine && cntrk, "goff" );
  for(Long64_t i=0; i<h_evt->GetNbinsX(); ++i) {
    // Exactly one track in kinematics window
    if( h_evt->GetBinContent(i)>0.1 &&
	h_evt->GetBinContent(i)<1.1 )
      ++c_kine;
  }
  // This is what would probably be called a "good" or "unambiguous" event
  // in the analysis of real data: exactly one track within the kine window
  cout << "Events with ==1 track in kinematics window = " << c_kine << endl;
  // Fraction of single tracks in kine window that DON'T match the real track
  cout << "Mis-reconstructed fraction = " 
       << double(c_kine-c_single)/c_kine*100.  << "%" << endl;

  //  T->Draw("SBS.tr.x:SBS.tr.y", ctrk && cntrk);
  //  gPad->SaveAs(Form("%s_trx_try.png",inp.c_str()));

  // cMCkine: MC truth track within kinematics window...apparently not always!
  TCut cMCkine = Form("abs(MC.tr.x)<%f && abs(MC.tr.y)<%f && "
		      "abs(MC.tr.th)<%f && abs(MC.tr.ph)<%f",
		      x_kine,y_kine,th_kine,ph_kine);
  //  cMCkine.Print();
  h_evt->Reset();
  UInt_t c_MCkine = 0;
  T->Draw("Entry$>>h_evt", cMCkine, "goff" );
  for(Long64_t i=0; i<h_evt->GetNbinsX(); ++i) {
    // Exactly one track in kinematics window
    if( h_evt->GetBinContent(i)>0.1 &&
	h_evt->GetBinContent(i)<1.1 )
      ++c_MCkine;
  }
  cout << "MC track in kine window = " << c_MCkine 
       << " (" << double(c_MCkine)/double(nevt)*100. << "%)"
       << endl;
  cout << "Recomputed ==1 track efficiency = " <<
    double(c_single)/double(c_MCkine)*100. << endl;    

  T->Draw("SBS.tr.x-MC.tr.x>>h1(100,-0.0005,0.0005)", cMCkine&&ckine&&cntrk&&"SBS.tr.n==1","goff");
  T->Draw("SBS.tr.y-MC.tr.y>>h2(100,-0.0005,0.0005)", cMCkine&&ckine&&cntrk&&"SBS.tr.n==1","goff");

  T->Draw("SBS.tr.th-MC.tr.th>>h3(100,-0.002,0.002)", cMCkine&&ckine&&cntrk&&"SBS.tr.n==1","goff");
  T->Draw("SBS.tr.ph-MC.tr.ph>>h4(100,-0.002,0.002)", cMCkine&&ckine&&cntrk&&"SBS.tr.n==1","goff");

  TCanvas *cc = new TCanvas("cc","",200,10,900,700);
  cc->Divide(2,2);

  cc->cd(1);
  h1->SetTitle("X residuals");
  h1->Fit("gaus");
  h1->Draw();

  cc->cd(2);
  h2->SetTitle("Y residuals");
  h2->Fit("gaus");
  h2->Draw(); 

  cc->cd(3);
  h3->SetTitle("#theta residuals");
  h3->Fit("gaus");
  h3->Draw();

  cc->cd(4);
  h4->SetTitle("#phi residuals");
  h4->Fit("gaus");
  h4->Draw();

  //  f->Close();
  //  delete f; f = 0;

}
